package com.oilabs.map.insuideindoor

import  com.oilabs.map.insuideindoor.ar.ar.WayfindingAr
//import com.oilabs.map.insuideindoor.ar.ar.*
//import WayfindingAr
import android.content.Context
import android.content.Intent
import android.util.Log
import androidx.annotation.NonNull
import androidx.core.content.ContextCompat.startActivity
import com.oilabs.map.insuideindoor.wayfinding.*
import com.indooratlas.android.sdk.IAPOI
import com.indooratlas.android.sdk.resources.IALatLng

import io.flutter.embedding.engine.plugins.activity.ActivityAware
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding
import io.flutter.embedding.engine.plugins.FlutterPlugin
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.plugin.common.MethodChannel.Result

/** InsuideindoorPlugin */
class InsuideindoorPlugin: FlutterPlugin, MethodCallHandler, ActivityAware {
  /// The MethodChannel that will the communication between Flutter and native Android
  ///
  /// This local reference serves to register the plugin with the Flutter Engine and unregister it
  /// when the Flutter Engine is detached from the Activity
  private lateinit var channel : MethodChannel
    private lateinit var _engine: IAFlutterEngine
    private var activityContext : Context? = null

  override fun onAttachedToEngine(@NonNull flutterPluginBinding: FlutterPlugin.FlutterPluginBinding) {
    activityContext =  flutterPluginBinding.applicationContext
    channel = MethodChannel(flutterPluginBinding.binaryMessenger, "insuideindoor")
    channel.setMethodCallHandler(this)
    _engine = IAFlutterEngine(activityContext!!, channel)
  }

  override fun onMethodCall(@NonNull call: MethodCall, @NonNull result: Result) {
    val res: Any = when (call.method) {
      "initialize" -> {
        _engine.initialize()
      }
      "lockIndoors" -> {
        _engine.lockIndoors((call.arguments as List<*>)[0] as Boolean?)
      }
      "lockFloors" -> {
        _engine.lockFloor(floor = (call.arguments as List<*>)[0] as Int)
      }
      "unlockFloors" -> {
        _engine.unlockFloor()
      }

      "setPositioningMode" -> {
        _engine.setPositioningMode((call.arguments as List<*>)[0] as Int)
      }

      "setOutputThresholds" -> {
        val list = call.arguments as List<*>
        _engine.setOutputThresholds(
                distance = list[0] as Double,
                interval = list[1] as Double
        )
      }
      "startPositioning" -> {
        _engine.startPositioning()
      }
      "stopPositioning" -> {
        _engine.stopPositioning()
      }
      "setSensitivities" -> {
        //val list = call.arguments as List<*>
//                _engine.setSensitivities(
//                    orientationSensitivity = list[0] as Double,
//                    headingSensitivity = list[1] as Double,
//                )
      }
      "startWayfinding" -> {
        val list = call.arguments as List<*>
        _engine.startWayfinding(
                list[0] as Double,
                list[1] as Double,
                list[2] as Int,
        )
      }
      "stopWayfinding" -> {
        _engine.stopWayfinding()
      }
      "getTraceId" -> {
        _engine.getTraceId()
      }
      "openAR" -> {
                // val list = call.arguments as List<*>
                // val intent = Intent(this@InsuideindoorPlugin,  WayfindingAr::class.java)
                // val poisMap = list[1] as List<Map<*,*>>

                // val pois = arrayListOf<IAPOI>()

                // for (map in poisMap) {
                //     pois.add(
                //         IAPOI(
                //             map["id"] as String,
                //             map["floor"] as Int,
                //             IALatLng(
                //                 (map["coordinate"] as Map<*, *>)["latitude"] as Double,
                //                 (map["coordinate"] as Map<*, *>)["latitude"] as Double
                //             ),
                //             map["name"] as String,
                //             null
                //         )
                //     )
                // }
                // intent.putExtra("selectedId", list[0] as String)
                // intent.putParcelableArrayListExtra("pois", pois)
                // startActivity( Intent(this@InsuideindoorPlugin, WayfindingAr::class.java) )
            }
      else -> {}
    }

    if (res is String) {
      result.success(res)
    }
  }

  override fun onDetachedFromEngine(@NonNull binding: FlutterPlugin.FlutterPluginBinding) {
    channel.setMethodCallHandler(null)
  }

  override fun onAttachedToActivity(binding: ActivityPluginBinding) {
    _engine.activityBinding = binding
  }

  override fun onDetachedFromActivityForConfigChanges() {
    _engine.activityBinding = null
  }

  override fun onReattachedToActivityForConfigChanges(binding: ActivityPluginBinding) {
    _engine.activityBinding = binding
  }

  override fun onDetachedFromActivity() {
    _engine.activityBinding = null
  }
}