package com.oilabs.map.insuideindoor.wayfinding

import android.content.Context
import android.os.Bundle
import android.util.Log
import android.os.Handler
import android.os.Looper
import com.oilabs.map.insuideindoor.utils.IALocation2Map
import com.oilabs.map.insuideindoor.utils.IARegion2Map
import com.oilabs.map.insuideindoor.utils.IARoute2Map
import com.indooratlas.android.sdk.*
import com.indooratlas.android.sdk.resources.IALatLng
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.PluginRegistry
import timber.log.Timber

class IAFlutterEngine(context: Context, channel: MethodChannel) : IALocationListener,
    IARegion.Listener, IAOrientationListener, IAWayfindingListener, PluginRegistry.RequestPermissionsResultListener {
    var activityBinding: ActivityPluginBinding? = null
        get() = field
        set(value) {
            if (field != null) {
                val old = field as ActivityPluginBinding
                old.removeRequestPermissionsResultListener(this)
            }
            if (value != null) {
                value.addRequestPermissionsResultListener(this)
            }
            field = value
        }
    private var _locationServiceRunning = false
    private val _handler = Handler(Looper.getMainLooper())
    private val _context: Context = context
    private val _channel: MethodChannel = channel
    private var _locationManager: IALocationManager? = null
    private var _locationRequest = IALocationRequest.create()
    private val _orientationRequest = IAOrientationRequest(1.0, -1.0)

    override fun onLocationChanged(location: IALocation) {
        _channel.invokeMethod("onLocationChanged", listOf(IALocation2Map(location)))
    }

    override fun onStatusChanged(provider: String, status: Int, bundle: Bundle?) {
        var mappedStatus = 0
        when (status) {
            IALocationManager.STATUS_OUT_OF_SERVICE -> {
                mappedStatus = 0
            }
            IALocationManager.STATUS_TEMPORARILY_UNAVAILABLE -> {
                mappedStatus = 1
            }
            IALocationManager.STATUS_AVAILABLE -> {
                mappedStatus = 2
            }
            IALocationManager.STATUS_LIMITED -> {
                mappedStatus = 3
            }
            else -> {
                // figure out how to return error
            }
        }
        _channel.invokeMethod("onStatusChanged", listOf(mappedStatus))
    }

    override fun onEnterRegion(region: IARegion) {
        _channel.invokeMethod("onEnterRegion", listOf(IARegion2Map(region)))
    }

    override fun onExitRegion(region: IARegion) {
        _channel.invokeMethod("onExitRegion", listOf(IARegion2Map(region)))
    }

    override fun onHeadingChanged(timestamp: Long, heading: Double) {
        _channel.invokeMethod("onHeadingChanged", listOf(timestamp, heading))
    }

    override fun onOrientationChange(timestamp: Long, quaternion: DoubleArray) {
//        _channel.invokeMethod(
//            "onOrientationChanged",
//            listOf(timestamp, quaternion[0], quaternion[1], quaternion[2], quaternion[3])
//        )
    }

    override fun onWayfindingUpdate(route: IARoute) {
        _channel.invokeMethod("onWayfindingUpdate", listOf(IARoute2Map(route)))
    }

    fun initialize() {
        Log.e("intialize IA","=>>>");
        if (_locationManager != null) {
            _locationManager?.destroy()
        }

        _locationServiceRunning = false
        _handler.post {
             Log.e("intialize IA 222222222222","=>>>");
            _locationManager = IALocationManager.create(_context)
        }
    }

    fun getTraceId(): String {
        if (_locationManager == null) return ""
//          store the traceid somewhere else and return that?
//                (we aren't calling this on main thread here maybe)
        return _locationManager?.extraInfo?.traceId ?: ""
    }

    fun lockIndoors(locked: Boolean?) {
        _handler.post {
            if (_locationManager != null) {
                _locationManager?.lockIndoors(locked!!)
            }
        }
    }

    fun lockFloor(floor: Int?) {
        _handler.post {
            if (_locationManager != null) {
                _locationManager?.lockFloor(floor!!)
            }
        }
    }

    fun unlockFloor() {
        _handler.post {
            if (_locationManager != null) {
                _locationManager?.unlockFloor()
            }
        }
    }

    fun setPositioningMode(mode: Int?) {
        var prio = 0
        when (mode) {
            0 -> prio = IALocationRequest.PRIORITY_HIGH_ACCURACY
            1 -> prio = IALocationRequest.PRIORITY_LOW_POWER
            2 -> prio = IALocationRequest.PRIORITY_CART_MODE
        }
        _locationRequest.priority = prio
    }

    fun setOutputThresholds(distance: Double?, interval: Double?) {
        if (distance!! < 0 || interval!! < 0) {
            // figure out how to return error
            return
        }
        val wasRunning = _locationServiceRunning
        if (wasRunning) stopPositioning()
        if (distance >= 0) _locationRequest.smallestDisplacement = distance.toFloat()
        if (interval >= 0) _locationRequest.fastestInterval =
            (interval * 1000 /* s -> ms */).toLong()
        if (wasRunning) startPositioning()
    }
//
//    fun setSensitivities(headingSensitivity: Double? = 1.0, orientationSensitivity: Double? = 0.0) {
//        _orientationRequest = IAOrientationRequest(headingSensitivity!!, orientationSensitivity!!)
//        _handler.post {
//            if (_locationManager != null) {
//                _locationManager?.unregisterOrientationListener(this)
//                _locationManager?.registerOrientationListener(_orientationRequest, this)
//            }
//        }
//    }

    fun startPositioning() {
        val lat = 19.098519
        val lng = 72.874318
        Log.e("startPositioning","="+lat)
        _handler.postDelayed({
            if (_locationManager != null) {
                _locationManager?.requestLocationUpdates(_locationRequest, this)
                _locationManager?.registerRegionListener(this)
                _locationManager?.setLocation(
                    IALocation.Builder()
                        .withLatitude(lat)
                        .withLongitude(lng)
                        .withAccuracy(10f)
                        .withFloorLevel(4)
                        .build()
                )
                // lockFloor(4)
                _locationManager?.registerOrientationListener(_orientationRequest, this)
                _locationServiceRunning = true
                Timber.tag("FAKELOC").d("%s, %s", lat, lng)
            }
        }, 2000)
    }

    fun stopPositioning() {
        _handler.post {
            if (_locationManager != null) {
                _locationManager?.removeLocationUpdates(this)
                _locationManager?.unregisterOrientationListener(this)
                _locationManager?.unregisterRegionListener(this)
                _locationServiceRunning = false
            }
        }
    }

    fun startWayfinding(lat: Double?, lon: Double?, floor: Int?) {
        _handler.post {
            _locationManager?.let {
                val request = IAWayfindingRequest.Builder().withLatitude(lat!!).withLongitude(lon!!)
                    .withFloor(floor!!).build()
                it.requestWayfindingUpdates(request, this)
            }
        }
    }

    fun stopWayfinding() {
        _handler.post {
            _locationManager?.removeWayfindingUpdates()
        }
    }

    private fun getPoIs(): List<IAPOI>? {
        val pois: MutableList<IAPOI> = ArrayList()
        pois.add(IAPOI("1", 1, IALatLng(19.09614069661849, 72.87494458659683), "Burger King", null))
        pois.add(IAPOI("2", 1, IALatLng(19.09930713759286, 72.87373404109417), "Shiv Sagar", null))
        pois.add(IAPOI("3", 1, IALatLng(19.09646722213212, 72.8744603204529), "Irani Cafe", null))
        return pois
    }

    override fun onRequestPermissionsResult(requestCode: Int, permissions: Array<out String>, grantResults: IntArray): Boolean {
        // TODO("Not yet implemented")
        return true
    }
}