// ignore_for_file: public_member_api_docs, sort_constructors_first
import 'dart:convert';

import 'package:equatable/equatable.dart';
import 'package:flutter/material.dart';

import 'ia_floorplan.dart';
import 'ia_point.dart';
import 'ia_region.dart';
import 'models.dart';

class IALocation extends IACoordinate {
  late final IAPoint? pixel;
  late final IAFloorplan? floorplan;
  final double accuracy;
  final double heading;
  final double altitude;
  final int floor;
  final double floorCertainty;
  final double velocity;
  final DateTime timestamp;
  IALocation({
    required double latitude,
    required double longitude,
    this.accuracy = 0,
    this.heading = 0,
    this.altitude = 0,
    this.floor = 0,
    this.floorCertainty = 0,
    this.velocity = 0,
    required this.timestamp,
  })  : pixel = null,
        floorplan = null,
        super(latitude, longitude);
  IALocation.fromCoordinate(
    IACoordinate coordinate, {
    this.accuracy = 0,
    this.heading = 0,
    this.altitude = 0,
    this.floor = 0,
    this.floorCertainty = 0,
    this.velocity = 0,
    required this.timestamp,
  })  : pixel = null,
        floorplan = null,
        super(coordinate.latitude, coordinate.longitude);
  IALocation.fromMap(Map map)
      : accuracy = map['accuracy'],
        heading = map['heading'],
        altitude = map['altitude'],
        floor = map['flr'],
        floorCertainty = map['floorCertainty'],
        velocity = map['velocity'],
        timestamp = DateTime.fromMillisecondsSinceEpoch(map['timestamp']),
        super(map['latitude'], map['longitude']) {
    if (map.containsKey('region')) {
      this.floorplan = Region.fromMap(map['region']).floorplan;
    }
    if (map.containsKey('pix_x') && map.containsKey('pix_y')) {
      this.pixel = IAPoint(map['pix_x'], map['pix_y']);
    }
  }
}
