import 'package:flutter/material.dart';
import 'package:flutter/src/widgets/container.dart';
import 'package:flutter/src/widgets/framework.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:insuideindoor/data/ia/indooratlas.dart';
import 'package:insuideindoor/data/models/ia_wayfindingrequest.dart';
import 'package:insuideindoor/data/models/models.dart';
import 'package:insuideindoor/main/MyHome.dart';
import 'package:insuideindoor/main/bloc/myhome_bloc.dart';
import 'package:insuideindoor/main/common_widget.dart';
import 'package:insuideindoor/main/poi_model.dart';
import 'package:insuideindoor/ui/widgets/ia_listener.dart';
import 'package:insuideindoor/utils/common_functions.dart';
import 'package:mapbox_gl/mapbox_gl.dart';

import '../data/models/ia_coordinate.dart';

class Direction extends StatefulWidget {
  final String kamalString;
  final String toText;
  final double toLat;
  final double toLng;
  final String iconImage;

  const Direction(
      {super.key,
      required this.kamalString,
      required this.toText,
      required this.toLat,
      required this.iconImage,
      required this.toLng});

  @override
  State<Direction> createState() => _DirectionState();
}

class _DirectionState extends State<Direction> {
  TextEditingController fromController = TextEditingController();
  TextEditingController toController = TextEditingController();
  late MapboxMapController _mapBoxController;
  List<IAFloorplan>? _floorplans;

  void addSymbolD(MapboxMapController mapBoxController) async {
    //  SHOWING CURRENT USER LOCATION ON MAP
    await mapBoxController.addSymbol(
      const SymbolOptions(
        geometry: LatLng(19.098519, 72.874318),
        iconImage: "assets/symbols/Oval.png",
        iconSize: 2,
      ),
    );
    mapBoxController.addSymbol(
      SymbolOptions(
          geometry: LatLng(widget.toLat, widget.toLng),
          iconImage: CommonFunctions.iconReturn(widget.iconImage),
          iconSize: 2.5,
          textField: widget.toText,
          textSize: 15,
          iconOffset: const Offset(0, -10)),
    );
  }

  @override
  void initState() {
    // TODO: implement initState
    super.initState();
    fromController.text = "Your Location";
    IndoorAtlas.startWayfindingTo(
      IAWayfindingRequest(
        destination: IACoordinate(widget.toLat, widget.toLng),
        floor: 2,
      ),
    );
  }

  @override
  void dispose() {
    // TODO: implement dispose
    super.dispose();
  }

  Line? directionLine;
  bool startVisible = true;
  Widget build(BuildContext context) {
    @override
    void didUpdateWidget(Direction oldWidget) {
      super.didUpdateWidget(oldWidget);
    }

    void _onMapCreated(MapboxMapController mapboxMapController) {
      _mapBoxController = mapboxMapController;
    }

    void _onFloorPlanD(bool enter, IAFloorplan floorplan) {
      // if (floorplan.id != floorplans![currentFloor].id) {
      //   _changeFloorPlanImage(floorplan);
      // }
    }

    void _onHeadingD(double value) {}
    // MapboxMapController? mapBoxGlobalController;
    Symbol? userLocSymbol;
    void _onLocationD(IALocation value) async {
      print('LOCATION CHANGED ');
      print(value.latitude);
      print(value.longitude);
      // if (userLocSymbol != null) {
      _mapBoxController.removeSymbol(userLocSymbol!);
      // }
      ScaffoldMessenger.of(context).showSnackBar(
        SnackBar(
          content: Text(
              " ${value.latitude.toString()}, ${value.longitude.toString()}"),
        ),
      );

      await _mapBoxController.addSymbol(
        SymbolOptions(
          geometry: LatLng(value.latitude, value.longitude),
          iconImage: "assets/symbols/Oval.png",
          iconSize: 2,
        ),
      );
      await _mapBoxController.animateCamera(
        CameraUpdate.newLatLngZoom(
          LatLng(value.latitude, value.longitude),
          16.0,
        ),
      );
    }

    void _onVenueD(bool enter, IAVenue venue) {
      _floorplans = venue.floorplans;
    }

    void _onOrientationChangedD(double x, double y, double z, double w) {}

    void _onStatusD(IAStatus status, String message) {}

    List<LatLng> _getRouteLatLng2(IARoute route) {
      print("GetRouteLatLng ${route.legs}");
      List<LatLng> geometry = [];
      for (var leg in route.legs) {
        print(" GetRouteLatLng66 ${leg.end.coordinate.latitude}");
        geometry
          ..add(
            LatLng(
                leg.begin.coordinate.latitude, leg.begin.coordinate.longitude),
          )
          ..add(
            LatLng(leg.end.coordinate.latitude, leg.end.coordinate.longitude),
          );
      }
      return geometry;
    }

    void _onWayfindingRouteD(
        IARoute route, IAWayfindingRequest? request) async {
      await _mapBoxController.clearLines();

      directionLine = await _mapBoxController.addLine(
        LineOptions(
          geometry: _getRouteLatLng2(route),
          lineWidth: 5,
          lineColor: '#006DB7',
        ),
      );
      setState(() {});
    }

    double width = MediaQuery.of(context).size.width;
    double height = MediaQuery.of(context).size.height;
    FocusNode fromFocus = FocusNode();
    FocusNode toFocus = FocusNode();
    bool canCloseFrom = false;
    bool canCloseTo = false;
    // double btnPadding = 10.0;
    double btnSize = 25.0;
    double fontSize = 16.0;
    var bordrWdth = 2.0;
    return Scaffold(
      bottomNavigationBar: startVisible
          ? Container(
              decoration: const BoxDecoration(
                color: Colors.white,
                boxShadow: <BoxShadow>[
                  BoxShadow(
                      color: Colors.black54,
                      blurRadius: 8.0,
                      offset: Offset(0.0, 0.75))
                ],
              ),
              height: 100,
              child: Padding(
                padding: const EdgeInsets.all(8.0),
                child: Row(
                  mainAxisAlignment: MainAxisAlignment.spaceBetween,
                  crossAxisAlignment: CrossAxisAlignment.center,
                  children: [
                    Row(
                      // mainAxisAlignment: MainAxisAlignment.center,
                      // crossAxisAlignment: CrossAxisAlignment.center,
                      children: [
                        const Icon(Icons.directions_walk_outlined),
                        Column(
                          mainAxisAlignment: MainAxisAlignment.center,
                          crossAxisAlignment: CrossAxisAlignment.start,
                          children: const [
                            Text(
                              "12 Mins",
                              style: TextStyle(
                                  fontSize: 22, fontWeight: FontWeight.bold),
                            ),
                            Text("500m")
                          ],
                        )
                      ],
                    ),
                    commonBtn(() {
                      setState(() {
                        startVisible = !startVisible;
                        print("value updated");
                      });
                    },
                        Colors.blue,
                        Row(
                          children: const [
                            Text(
                              "Start",
                              style: TextStyle(fontSize: 20),
                            ),
                            Icon(Icons.navigation_outlined)
                          ],
                        ))
                  ],
                ),
              ),
            )
          : Container(
              decoration: const BoxDecoration(
                color: Colors.white,
                boxShadow: <BoxShadow>[
                  BoxShadow(
                      color: Colors.black54,
                      blurRadius: 8.0,
                      offset: Offset(0.0, 0.75))
                ],
              ),
              height: 140,
              // color: Colors.white,
              child: Padding(
                padding: const EdgeInsets.all(8.0),
                child: Column(
                  mainAxisAlignment: MainAxisAlignment.center,
                  crossAxisAlignment: CrossAxisAlignment.start,
                  children: [
                    const Text(
                      "12 Mins",
                      style:
                          TextStyle(fontSize: 22, fontWeight: FontWeight.bold),
                    ),
                    Row(
                      children: const [
                        Text("500m"),
                        Text(" • "),
                        Text("11:25 AM")
                      ],
                    ),
                    const SizedBox(height: 10),
                    Row(
                      mainAxisAlignment: MainAxisAlignment.spaceBetween,
                      children: [
                        commonBtn(() {
                          _mapBoxController.removeLine(directionLine!);
                          setState(() {
                            startVisible = !startVisible;
                          });
                          Navigator.push(
                            context,
                            MaterialPageRoute(
                                builder: (context) => BlocProvider(
                                      create: (context) => MyhomeBloc(),
                                      child: MyHome(
                                        data:
                                            "26.828943333333335,75.80561666666667",
                                        mTer: '2',
                                        mLev: '4',
                                      ),
                                    )),
                          );
                        },
                            Colors.black,
                            const Text(
                              "Exit",
                              style: TextStyle(fontSize: 20),
                            )),
                        commonBtn(
                            () {},
                            Colors.white,
                            Row(
                              children: const [
                                Text(
                                  "AR View  ",
                                  style: TextStyle(
                                      fontSize: 18, color: Colors.black),
                                ),
                                Icon(
                                  Icons.looks_sharp,
                                  color: Colors.black,
                                )
                              ],
                            )),
                      ],
                    )
                  ],
                ),
              ),
            ),
      body: SafeArea(
        child: Stack(
          // alignment: Alignment.bottomCenter,
          children: [
            Align(
              alignment: Alignment.bottomCenter,
              child: SizedBox(
                height: startVisible ? height * 0.7 : height,
                child: IndoorAtlasListener(
                  name: 'insuide',
                  onFloorplan: _onFloorPlanD,
                  onHeading: _onHeadingD,
                  onLocation: _onLocationD,
                  onVenue: _onVenueD,
                  onOrientation: _onOrientationChangedD,
                  onStatus: _onStatusD,
                  onWayfindingRoute: _onWayfindingRouteD,
                  child: MapboxMap(
                    tiltGesturesEnabled: true,
                    accessToken:
                        "sk.eyJ1IjoibWVycmlsbGdvbnNhbHZlcyIsImEiOiJja2l6eno3ZWk0ZnYxMnNsYng5azdxdjBiIn0.fkx2_7LZ7uq80mHYCuh46w",
                    initialCameraPosition: const CameraPosition(
                        target: LatLng(19.098519, 72.874318),
                        zoom: 17,
                        tilt: 30.0),
                    onMapCreated: _onMapCreated,
                    trackCameraPosition: true,
                    onCameraTrackingChanged: (mode) {},
                    onStyleLoadedCallback: () => addSymbolD(_mapBoxController),
                    myLocationTrackingMode: MyLocationTrackingMode.TrackingGPS,
                    styleString: widget.kamalString,
                  ),
                ),
              ),
            ),
            startVisible
                ? GestureDetector(
                    onTap: () {
                      Navigator.pop(context);
                    },
                    child: Container(
                      height: startVisible ? height * 0.15 : height * 0.16,
                      width: width,
                      margin: const EdgeInsets.symmetric(horizontal: 15),
                      child: Stack(
                        children: [
                          Column(
                            crossAxisAlignment: CrossAxisAlignment.end,
                            children: [
                              Row(
                                crossAxisAlignment: CrossAxisAlignment.center,
                                mainAxisAlignment:
                                    MainAxisAlignment.spaceBetween,
                                children: [
                                  GestureDetector(
                                    onTap: (() {
                                      Navigator.pop(context);
                                    }),
                                    child: Icon(
                                      Icons.chevron_left,
                                      color: Colors.black,
                                      size: width * 0.1,
                                    ),
                                  ),
                                  //FROM
                                  Container(
                                    width: width * 0.88 - 30,
                                    // color: Colors.teal,
                                    alignment: Alignment.centerLeft,
                                    child: TextFormField(
                                      onTap: () {
                                        Navigator.pop(context);
                                      },
                                      readOnly: true,
                                      focusNode: fromFocus,
                                      controller: fromController,
                                      scrollPadding: EdgeInsets.zero,
                                      // textAlignVertical: TextAlignVertical.center,
                                      decoration: InputDecoration(
                                        isDense: true,
                                        // contentPadding: const EdgeInsets.only(top: 0.0, bottom: 0.0, left: 0.0, right: 5.0),
                                        contentPadding: EdgeInsets.zero,
                                        prefixIcon: Icon(Icons.circle_outlined,
                                            size: btnSize,
                                            color: Color(0xFF555555)),

                                        hintText: "From",
                                        hintStyle: const TextStyle(
                                            fontWeight: FontWeight.bold,
                                            fontSize: 15),
                                        border: OutlineInputBorder(
                                            borderRadius:
                                                BorderRadius.circular(40),
                                            // borderSide: BorderSide(color: Color(0xffEAEBED)
                                            borderSide: BorderSide(
                                                color: Color(0xffCACACA),
                                                width: bordrWdth)),
                                        disabledBorder: OutlineInputBorder(
                                            borderRadius:
                                                BorderRadius.circular(40),
                                            // borderSide: BorderSide(color: Color(0xffEAEBED)
                                            borderSide: BorderSide(
                                                color: Color(0xffCACACA),
                                                width: bordrWdth)),
                                        enabledBorder: OutlineInputBorder(
                                            borderRadius:
                                                BorderRadius.circular(40),
                                            borderSide: BorderSide(
                                                color: Color(0xffEAEBED),
                                                width: bordrWdth)),
                                        focusedBorder: OutlineInputBorder(
                                            borderRadius:
                                                BorderRadius.circular(40),
                                            borderSide: BorderSide(
                                                color: Color(0xffEAEBED),
                                                width: bordrWdth)),
                                      ),
                                      onChanged: (text) {
                                        if (text.isNotEmpty) {
                                          searchList.clear();
                                          for (int i = 0;
                                              i < dataPois.length;
                                              i++) {
                                            Datum d = dataPois[i];

                                            if (d.category
                                                    .toString()
                                                    .toLowerCase()
                                                    .contains(text) ||
                                                d.subcategory
                                                    .toString()
                                                    .toLowerCase()
                                                    .contains(text) ||
                                                d.poiMultilingual!.en!.name
                                                    .toString()
                                                    .toLowerCase()
                                                    .contains(text)) {
                                              searchList.add(d);
                                            }
                                          }

                                          if (!canCloseFrom) {
                                            canCloseFrom = true;
                                          }
                                        } else {
                                          searchList.clear();
                                          if (canCloseFrom) {
                                            canCloseFrom = false;
                                          }
                                        }
                                        setState(() {});
                                      },
                                    ),
                                  ),
                                ],
                              ),

                              Container(
                                color: Colors.white,
                                height: 20,
                              ),
                              //TO
                              SizedBox(
                                width: width * 0.88 - 30,
                                child: TextFormField(
                                  onTap: () {
                                    Navigator.pop(context);
                                  },
                                  initialValue: widget.toText,
                                  readOnly: true,
                                  focusNode: toFocus,
                                  // controller: toController,
                                  scrollPadding: EdgeInsets.zero,
                                  decoration: InputDecoration(
                                    // fillColor: Color(0xffEAEBED),
                                    isDense: true,
                                    contentPadding: const EdgeInsets.only(
                                        top: 0.0,
                                        bottom: 0.0,
                                        left: 0.0,
                                        right: 5.0),
                                    prefixIcon: Icon(Icons.location_pin,
                                        size: btnSize,
                                        color: Color(0xFF555555)),
                                    suffix: canCloseTo
                                        ? GestureDetector(
                                            onTap: () {
                                              toController.text = "";
                                              // fromController.notifyListeners();
                                            },
                                            child: Icon(
                                              Icons.close,
                                              size: btnSize + 5,
                                              color: Color(0xFF555555),
                                            ),
                                          )
                                        : null,

                                    // hintText: "To",
                                    hintStyle: const TextStyle(
                                        fontWeight: FontWeight.bold,
                                        fontSize: 15),
                                    border: OutlineInputBorder(
                                        borderRadius: BorderRadius.circular(40),
                                        borderSide: BorderSide(
                                            color: Color(0xffCACACA),
                                            width: bordrWdth)),
                                    disabledBorder: OutlineInputBorder(
                                        borderRadius: BorderRadius.circular(40),
                                        borderSide: BorderSide(
                                            color: Color(0xffCACACA),
                                            width: bordrWdth)),
                                    enabledBorder: OutlineInputBorder(
                                        borderRadius: BorderRadius.circular(40),
                                        borderSide: BorderSide(
                                            color: Color(0xffEAEBED),
                                            width: bordrWdth)),
                                    focusedBorder: OutlineInputBorder(
                                        borderRadius: BorderRadius.circular(40),
                                        borderSide: BorderSide(
                                            color: Color(0xffEAEBED),
                                            width: bordrWdth)),
                                  ),
                                  // onTap: () {
                                  //   readOnly = !readOnly;
                                  //   setState(() {});
                                  // },

                                  // initialValue:
                                  //     searchController.text,
                                  onChanged: (text) {
                                    if (text.isNotEmpty) {
                                      searchList.clear();
                                      for (int i = 0;
                                          i < dataPois.length;
                                          i++) {
                                        Datum d = dataPois[i];

                                        if (d.category
                                                .toString()
                                                .toLowerCase()
                                                .contains(text) ||
                                            d.subcategory
                                                .toString()
                                                .toLowerCase()
                                                .contains(text) ||
                                            d.poiMultilingual!.en!.name
                                                .toString()
                                                .toLowerCase()
                                                .contains(text)) {
                                          searchList.add(d);
                                        }
                                      }

                                      if (!canCloseTo) {
                                        canCloseTo = true;
                                      }
                                    } else {
                                      searchList.clear();
                                      if (canCloseTo) {
                                        canCloseTo = false;
                                      }
                                    }
                                    setState(() {});
                                  },
                                  // autofocus: true,
                                  // readOnly: readOnly,
                                ),
                              ),
                            ],
                          ),
                          Container(
                            color: const Color(0xFF333333),
                            width: 1.5,
                            height: 35,
                            margin: EdgeInsets.only(
                                left: width * 0.11 + 26.75, top: 40),
                          ),
                        ],
                      ),
                    ),
                  )
                : Container()
          ],
        ),
      ),
    );
  }
}
