import 'dart:developer';

import 'package:flutter/material.dart';
import 'package:insuideindoor/data/ia/indooratlas.dart';
import 'package:insuideindoor/data/models/ia_coordinate.dart';
import 'package:insuideindoor/data/models/ia_wayfindingrequest.dart';
import 'package:insuideindoor/main/MyHome.dart';
import 'package:insuideindoor/main/poi_model.dart';
import 'package:insuideindoor/main/services.dart';
import 'package:mapbox_gl/mapbox_gl.dart';

class PoiInfo extends StatefulWidget {
  final String geo;
  final String poiName;
  final String iconImage;
  final bool isDetail;
  final Function? onGetDirection;
  final bool? fromFilterData;

  const PoiInfo(
      {Key? key,
      required this.geo,
      required this.poiName,
      required this.iconImage,
      this.onGetDirection,
      this.fromFilterData,
      this.isDetail = true})
      : super(key: key);

  @override
  State<PoiInfo> createState() =>
      isDetail ? _PoiInfoDetailState() : _PoiInfoState();
}

class _PoiInfoState extends State<PoiInfo> {
  bool isLoading = true;
  var reqData;

  String? pImage;
  String? pMobile;
  String? pDec;
  String? pTerminal;
  String? pName;

  // fetchPois() async {
  //   GetPoi? poiInfoData = await InternetService().getPOI().then((value) {
  //     isLoading = false;

  //     String geoo = widget.geo.toString();
  //     List first = geoo.split("(");
  //     List second = first[1].toString().split(",");
  //     for (int a = 1; a <= loopCounter!; a++) {
  //       if (value!.data![a - 1].lat.toString().contains(second[0]) &&
  //           value.data?[a - 1].poiMultilingual?.en?.name == widget.poiName) {
  //         setState(() {
  //           pName = value.data?[a - 1].poiMultilingual?.en?.name;
  //           pTerminal = value.data?[a - 1].terminal;
  //           pDec = value.data?[a - 1].poiMultilingual!.en!.description;
  //           pMobile = value.data?[a - 1].mobileNumber;
  //           pImage = value.data![a - 1].poiUrl;
  //         });
  //         print("pImage ${value.data![a - 1].poiUrl}");
  //         print(
  //             "FINAL DATA ${reqData = value.data?[a - 1].poiMultilingual?.en?.name}");
  //       } else {
  //         continue;
  //       }
  //     }
  //     return null;
  //   });
  // }

  @override
  void initState() {
    super.initState();
    // fetchPois();
    filterPOI();
  }

  filterPOI() {
    String geoo = widget.geo.toString();
    List first = geoo.split("(");
    List second = first[1].toString().split(",");

    int loopCounter = InternetService.poiData!.data!.length;
    var data = InternetService.poiData!.data;
    for (int a = 1; a <= loopCounter; a++) {
      var item = data![a - 1];
      if (item.lat.toString().contains(second[0]) &&
          item.poiMultilingual?.en?.name == widget.poiName) {
        setState(() {
          pName = item.poiMultilingual?.en?.name;
          pTerminal = item.terminal;
          pDec = item.poiMultilingual!.en!.description;
          pMobile = item.mobileNumber;
          pImage = item.poiUrl;
        });
        print("pImage ${data[a - 1].poiUrl}");
        print("FINAL DATA ${reqData = data[a - 1].poiMultilingual?.en?.name}");
      } else {
        continue;
      }
    }
  }

  @override
  Widget build(BuildContext context) {
    double width = MediaQuery.of(context).size.width;

    return Padding(
        padding: const EdgeInsets.fromLTRB(15, 0, 15, 8),
        child: Column(
          mainAxisAlignment: MainAxisAlignment.start,
          crossAxisAlignment: CrossAxisAlignment.start,
          children: [
            const SizedBox(height: 10),

            Align(
              alignment: Alignment.center,
              child: Container(
                width: width * 0.2,
                height: 5,
                decoration: BoxDecoration(
                    color: const Color(0xffEAEBED),
                    borderRadius: BorderRadius.circular(10.0)),
              ),
            ),
            const SizedBox(height: 15),
            // SizedBox(height: MediaQuery.of(context).size.height / ),

            Row(
              mainAxisAlignment: MainAxisAlignment.start,
              children: [
                CircleAvatar(
                    backgroundColor: Colors.white,
                    radius: 30,
                    backgroundImage: AssetImage(widget.iconImage)),
                const SizedBox(width: 15),
                Expanded(
                  child: Column(
                    crossAxisAlignment: CrossAxisAlignment.start,
                    children: [
                      Text(
                        pName ?? "Loading",
                        // overflow: TextOverflow.visible,
                        softWrap: true,
                        style: const TextStyle(
                            fontWeight: FontWeight.bold, fontSize: 25),
                      ),
                      const SizedBox(height: 5),
                      // Row(
                      //   children: const [
                      //     Text("Address"),
                      //     Text(" "),
                      //     Text("1 km Away")
                      //   ],
                      // ),
                      // const SizedBox(height: 5),
                      Text("Terminal $pTerminal")
                    ],
                  ),
                )
              ],
            ),
            const SizedBox(height: 20),
            Container(
              height: 55,
              width: MediaQuery.of(context).size.width - 30,
              decoration: BoxDecoration(
                  color: Colors.white,
                  border: Border.all(color: Colors.black),
                  borderRadius: const BorderRadius.all(Radius.circular(50))),
              child: GestureDetector(
                onTap: () {
                  // if (widget.onGetDirection != null) {
                  //   widget.onGetDirection!();
                  //   log("OnWayfindingClicked", name: "Mohit");
                  // }
                },
                child: Row(
                  mainAxisAlignment: MainAxisAlignment.center,
                  children: [
                    const Text(
                      "Get Directions",
                      style: TextStyle(
                          color: Colors.black,
                          fontSize: 18,
                          fontWeight: FontWeight.w600),
                    ),
                    const SizedBox(width: 15),
                    Image.asset(
                      "assets/images/location.png",
                      height: 20,
                      width: 20,
                    ),
                  ],
                ),
              ),
            ),
            const SizedBox(height: 20),

            (pImage == null || pImage == "")
                ? Container()
                : ClipRRect(
                    borderRadius: BorderRadius.circular(8.0),
                    child: Image.network(pImage!)),
            const SizedBox(height: 5),
            Text(
              "Description >> $pDec",
              style: const TextStyle(color: Color(0xFF222222)),
            ),
            const SizedBox(height: 5),

            const Text(
              "Information",
              style: TextStyle(fontSize: 17, fontStyle: FontStyle.italic),
            ),
            const SizedBox(height: 20),
            Row(
              children: [
                const Icon(Icons.phone_outlined),
                Text(pMobile ?? "No Number")
              ],
            ),
            const SizedBox(height: 20),
            Row(
              children: const [
                Icon(Icons.watch_later_outlined),
                Text("5 AM -10 PM")
              ],
            ),

            // Container(
            //   height: 90,
            //   color: Colors.deepPurple,
            // )
          ],
          // ),
        )
        // ),
        //   ),
        // ),
        //     )
        // )
        );
  }
}

class _PoiInfoDetailState extends State<PoiInfo> {
  bool isLoading = true;
  var reqData;

  String? pImage;
  String? pMobile;
  String? pDec;
  String? pTerminal;
  String? pName;

  // fetchPois() async {
  //   GetPoi? poiInfoData = await InternetService().getPOI().then((value) {
  //     isLoading = false;

  //     String geoo = widget.geo.toString();
  //     List first = geoo.split("(");
  //     List second = first[1].toString().split(",");
  //     for (int a = 1; a <= loopCounter!; a++) {
  //       if (value!.data![a - 1].lat.toString().contains(second[0]) &&
  //           value.data?[a - 1].poiMultilingual?.en?.name == widget.poiName) {
  //         setState(() {
  //           pName = value.data?[a - 1].poiMultilingual?.en?.name;
  //           pTerminal = value.data?[a - 1].terminal;
  //           pDec = value.data?[a - 1].poiMultilingual!.en!.description;
  //           pMobile = value.data?[a - 1].mobileNumber;
  //           pImage = value.data![a - 1].poiUrl;
  //         });
  //         print("pImage ${value.data![a - 1].poiUrl}");
  //         print(
  //             "FINAL DATA ${reqData = value.data?[a - 1].poiMultilingual?.en?.name}");
  //       } else {
  //         continue;
  //       }
  //     }
  //     return null;
  //   });
  // }

  @override
  void initState() {
    super.initState();
    // fetchPois();
    filterPOI();
    getLatLng();
  }

  String? lat;
  String? lng;
  getLatLng() {
    String geoo = widget.geo;
    List first = geoo.split("LatLng(");

    List second = first[1].toString().split(",");
    List third = second[1].toString().split(")");
    setState(() {
      lat = second[0];
      lng = third[0];
    });
  }

  filterPOI() {
    String geoo = widget.geo.toString();
    List first = geoo.split("(");
    List second = first[1].toString().split(",");

    int loopCounter = InternetService.poiData!.data!.length;
    var data = InternetService.poiData!.data;
    for (int a = 1; a <= loopCounter; a++) {
      var item = data![a - 1];
      if (item.lat.toString().contains(second[0]) &&
          item.poiMultilingual?.en?.name == widget.poiName) {
        setState(() {
          pName = item.poiMultilingual?.en?.name;
          pTerminal = item.terminal;
          pDec = item.poiMultilingual!.en!.description;
          pMobile = item.mobileNumber;
          pImage = item.poiUrl;
        });
        print("pImage ${data[a - 1].poiUrl}");
        print("FINAL DATA ${reqData = data[a - 1].poiMultilingual?.en?.name}");
      } else {
        continue;
      }
    }
  }

  @override
  Widget build(BuildContext context) {
    double width = MediaQuery.of(context).size.width;

    return SafeArea(
        child: Scaffold(
            body: Padding(
                padding: const EdgeInsets.fromLTRB(15, 0, 15, 8),
                child: Column(
                  mainAxisAlignment: MainAxisAlignment.start,
                  crossAxisAlignment: CrossAxisAlignment.start,
                  children: [
                    const SizedBox(height: 10),
                    // Align(
                    //   alignment: Alignment.center,
                    //   child: Container(
                    //     width: width * 0.2,
                    //     height: 5,
                    //     decoration: BoxDecoration(
                    //         color: const Color(0xffEAEBED),
                    //         borderRadius: BorderRadius.circular(10.0)),
                    //   ),
                    // ),
                    GestureDetector(
                        onTap: () {
                          Navigator.pop(context);
                        },
                        child: const Icon(Icons.arrow_back_ios)),
                    const SizedBox(height: 15),
                    // SizedBox(height: MediaQuery.of(context).size.height / ),

                    Row(
                      mainAxisAlignment: MainAxisAlignment.start,
                      children: [
                        CircleAvatar(
                            backgroundColor: Colors.white,
                            radius: 30,
                            backgroundImage: AssetImage(widget.iconImage)),
                        const SizedBox(width: 15),
                        Expanded(
                          child: Column(
                            crossAxisAlignment: CrossAxisAlignment.start,
                            children: [
                              Text(
                                pName ?? "Loading",
                                // overflow: TextOverflow.visible,
                                softWrap: true,
                                style: const TextStyle(
                                    fontWeight: FontWeight.bold, fontSize: 25),
                              ),
                              const SizedBox(height: 5),
                              // Row(
                              //   children: const [
                              //     Text("Address"),
                              //     Text(" "),
                              //     Text("1 km Away")
                              //   ],
                              // ),
                              // const SizedBox(height: 5),
                              Text("Terminal $pTerminal")
                            ],
                          ),
                        )
                      ],
                    ),
                    const SizedBox(height: 20),
                    Container(
                      height: 55,
                      width: MediaQuery.of(context).size.width - 30,
                      decoration: BoxDecoration(
                          color: Colors.white,
                          border: Border.all(color: Colors.black),
                          borderRadius:
                              const BorderRadius.all(Radius.circular(50))),
                      child: GestureDetector(
                        onTap: () {
                          print("lat $lat");
                          print("lat $lng");

                          IndoorAtlas.startWayfindingTo(
                            IAWayfindingRequest(
                              destination: IACoordinate(
                                  double.parse(lat!), double.parse(lng!)),
                              floor: 2,
                            ),
                          );

                          Navigator.pop(context);
                          // if (widget.fromFilterData ?? false) {
                          //   Navigator.pop(context);
                          // }
                        },
                        child: Row(
                          mainAxisAlignment: MainAxisAlignment.center,
                          children: [
                            const Text(
                              "Get Directions",
                              style: TextStyle(
                                  color: Colors.black,
                                  fontSize: 18,
                                  fontWeight: FontWeight.w600),
                            ),
                            const SizedBox(width: 15),
                            Image.asset(
                              "assets/images/location.png",
                              height: 20,
                              width: 20,
                            ),
                          ],
                        ),
                      ),
                    ),
                    const SizedBox(height: 20),

                    (pImage == null || pImage == "")
                        ? Container()
                        : ClipRRect(
                            borderRadius: BorderRadius.circular(8.0),
                            child: Image.network(pImage!)),
                    const SizedBox(height: 5),
                    Text(
                      "Description >> $pDec",
                      style: const TextStyle(color: Color(0xFF222222)),
                    ),
                    const SizedBox(height: 5),

                    const Text(
                      "Information",
                      style:
                          TextStyle(fontSize: 17, fontStyle: FontStyle.italic),
                    ),
                    const SizedBox(height: 20),
                    Row(
                      children: [
                        const Icon(Icons.phone_outlined),
                        Text(pMobile ?? "No Number")
                      ],
                    ),
                    const SizedBox(height: 20),
                    Row(
                      children: const [
                        Icon(Icons.watch_later_outlined),
                        Text("5 AM -10 PM")
                      ],
                    ),

                    // Container(
                    //   height: 90,
                    //   color: Colors.deepPurple,
                    // )
                  ],
                  // ),
                )
                // ),
                //   ),
                // ),
                )));
  }
}
