import 'dart:developer';

import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:insuideindoor/data/ia/indooratlas.dart';
import 'package:insuideindoor/data/models/models.dart';
import 'package:insuideindoor/ui/pages/home/home_page.dart';
import 'package:insuideindoor/ui/widgets/ia_listener.dart';
import 'package:insuideindoor/utils/constants.dart';
import '../../../utils/helper.dart';
import 'package:mapbox_gl/mapbox_gl.dart';
import 'package:http/http.dart' as http;
import 'bloc/mapbox_gl_bloc.dart';

class MapboxGLImpl extends StatefulWidget {
  const MapboxGLImpl({super.key});

  @override
  State<MapboxGLImpl> createState() => _MapboxGLImplState();
}

class _MapboxGLImplState extends State<MapboxGLImpl> {
  final _kInitialPosition = const CameraPosition(
    target: LatLng(19.09614069661849, 72.87494458659683),
    zoom: 13,
  );

  MapboxMapController? mapController;
  final MinMaxZoomPreference _minMaxZoomPreference =
      const MinMaxZoomPreference(5, 22);
  int currentFloor = 0;
  List<IAFloorplan>? floorplans;
  String currentStyle = mapboxStyles[0];
  int currentIndex = 0;
  final isDialOpen = ValueNotifier(false);
  String? selectedPoiForAR;
  List<IAPOI>? allPOIS;

  @override
  void initState() {
    super.initState();
    allPOIS = getPOIS();
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: AppBar(
        title: const Text('MapBoxGL'),
      ),
      // floatingActionButton: SpeedDial(
      //   icon: Icons.change_circle,
      //   activeIcon: Icons.close,
      //   spacing: 12,
      //   spaceBetweenChildren: 12,
      //   children: [
      //     SpeedDialChild(
      //       child: const Icon(Icons.change_circle),
      //       label: 'Change Style',
      //       onTap: () async {
      //         changeStyle();
      //       },
      //     ),
      //     SpeedDialChild(
      //       child: const Icon(Icons.change_circle),
      //       label: 'AR Mode',
      //       onTap: () async {
      //         try {
      //           final poisMaps = allPOIS!.map((poi) => poi.toMap()).toList();

      //           await methodChannel.invokeMethod(
      //             'openAR',
      //             [
      //               selectedPoiForAR ?? allPOIS![0].id,
      //               poisMaps,
      //             ],
      //           );
      //         } on PlatformException catch (e) {
      //           log("${e.message}");
      //         }
      //       },
      //     ),
      //   ],
      // ),

      floatingActionButton: FloatingActionButton(
        onPressed: () {
          () async {
            try {
              final poisMaps = allPOIS!.map((poi) => poi.toMap()).toList();
              await methodChannel.invokeMethod(
                'openAR',
                [
                  selectedPoiForAR ?? allPOIS![0].id,
                  poisMaps,
                ],
              );
            } on PlatformException catch (e) {
              log("${e.message}");
            }
          };
        },
      ),
      body: SafeArea(
        child: IndoorAtlasListener(
          name: 'IAMapboxGL',
          onFloorplan: _onFloorPlan,
          onHeading: _onHeading,
          onLocation: _onLocation,
          onVenue: _onVenue,
          onOrientation: _onOrientationChanged,
          onStatus: _onStatus,
          onWayfindingRoute: _onWayfindingRoute,
          child: BlocBuilder<MapboxGlBloc, MapboxGlState>(
            bloc: BlocProvider.of<MapboxGlBloc>(context),
            builder: (context, state) {
              if (state is OnChangeMapStyle) {
                currentStyle = state.style;
              }
              return MapboxMap(
                accessToken:
                    "sk.eyJ1IjoibWVycmlsbGdvbnNhbHZlcyIsImEiOiJja2l6eno3ZWk0ZnYxMnNsYng5azdxdjBiIn0.fkx2_7LZ7uq80mHYCuh46w",
                trackCameraPosition: true,
                initialCameraPosition: _kInitialPosition,
                styleString: currentStyle,
                rotateGesturesEnabled: true,
                minMaxZoomPreference: _minMaxZoomPreference,
                onMapCreated: _onMapCreated,
                myLocationEnabled: true,
                myLocationTrackingMode: MyLocationTrackingMode.TrackingGPS,
                myLocationRenderMode: MyLocationRenderMode.GPS,
                onStyleLoadedCallback: _onStyleLoaded,
                annotationOrder: const [
                  AnnotationType.fill,
                  AnnotationType.circle,
                  AnnotationType.line,
                  AnnotationType.symbol,
                ],
              );
            },
          ),
        ),
      ),
    );
  }

  void changeStyle() {
    BlocProvider.of<MapboxGlBloc>(context).add(ChangeMapStyle(currentIndex));
    currentIndex++;
  }

  void _onFloorPlan(bool enter, IAFloorplan? floorplan) async {
    if (floorplan != null) {
      if (floorplan.id != floorplans![currentFloor].id) {
        _changeFloorPlanImage(floorplan);
      }
      log('ONFLOORPLAN = $floorplan');
    }
  }

  Future _changeFloorPlanImage(IAFloorplan floorplan) async {
    await mapController!.removeSource('floorPlanSrc');
    final response = await http.get(Uri.parse(floorplan.url));
    await mapController!.addImageSource(
      'floorPlanSrc',
      response.bodyBytes,
      LatLngQuad(
        topLeft: floorplan.topLeft.parseLatLng(),
        topRight: floorplan.topRight.parseLatLng(),
        bottomRight: floorplan.bottomRight.parseLatLng(),
        bottomLeft: floorplan.bottomLeft.parseLatLng(),
      ),
    );
    // await mapController!.addSource(
    //   'floorPlan',
    //   ImageSourceProperties(
    //     url: floorplan.url,
    //     coordinates: [
    //       [floorplan.topLeft.longitude, floorplan.topLeft.latitude],
    //       [floorplan.topRight.longitude, floorplan.topRight.latitude],
    //       [floorplan.bottomRight.longitude, floorplan.bottomRight.latitude],
    //       [floorplan.bottomLeft.longitude, floorplan.bottomLeft.latitude],
    //     ],
    //   ),
    // );
    log("CHANGE FLOOR PLAN");
    await mapController!.addImageLayerBelow('floorPlan', 'floorPlanSrc',
        mapController!.lineManager!.layerIds.first);
    await mapController!.animateCamera(
      CameraUpdate.newLatLngZoom(
        LatLng(floorplan.center.latitude, floorplan.center.longitude),
        16.0,
      ),
    );
    addPOIs(getPOIS());
  }

  void _onHeading(double heading) {}

  void _onLocation(IALocation location) {
    currentFloor = location.floor;
    log('CurrentFloor = $currentFloor');
  }

  void _onMapCreated(MapboxMapController controller) {
    mapController = controller;
    //controller.addListener(onMapChanged);
    controller.onSymbolTapped.add(_onSymbolTapped);
  }

  List<IAPOI> getPOIS() {
    final List<IAPOI> pois = [];
    pois.addAll([
      IAPOI(
        id: '1',
        name: 'Burger King',
        floor: 2,
        coordinate: const IACoordinate(19.09614069661849, 72.87494458659683),
        payload: '',
      ),
      IAPOI(
        id: '2',
        name: 'Shiv Sagar',
        floor: 2,
        coordinate: const IACoordinate(19.09930713759286, 72.87373404109417),
        payload: '',
      ),
      IAPOI(
        id: '3',
        name: 'Irani Cafe',
        floor: 2,
        coordinate: const IACoordinate(19.09646722213212, 72.8744603204529),
        payload: '',
      ),
    ]);
    allPOIS = pois;
    return pois;
  }

  void addPOIs(List<IAPOI> pois) async {
    await mapController!.clearSymbols();
    for (var poi in pois) {
      await mapController!.addSymbol(
        SymbolOptions(
          iconImage: 'poi',
          geometry: LatLng(poi.coordinate.latitude, poi.coordinate.longitude),
          iconSize: 2.0,
          textField: poi.name,
        ),
      );
    }
    IndoorAtlas.startWayfindingTo(
      IAWayfindingRequest(
        destination: pois[2].coordinate,
        floor: currentFloor,
      ),
    );
    selectedPoiForAR = pois[2].id;
  }

  void onMapChanged() {}

  @override
  void dispose() {
    //mapController!.removeListener(onMapChanged);
    mapController!.onSymbolTapped.remove(_onSymbolTapped);
    super.dispose();
  }

  void _onVenue(bool enter, IAVenue venue) async {
    //log(venue.pois.toString(), name: "IAPOI");

    floorplans = venue.floorplans;
    log('ONVENUE', name: 'enter');
    _changeFloorPlanImage(floorplans![currentFloor]);
  }

  void _onOrientationChanged(double x, double y, double z, double w) {}

  void _onStatus(IAStatus status, String message) {}

  void _onWayfindingRoute(IARoute route, IAWayfindingRequest? request) async {
    log('ONWAYFINDINGROUTE: $route');
    await mapController?.clearLines();

    await mapController?.addLine(
      LineOptions(
        geometry: _getRouteLatLng(route),
        lineWidth: 5,
        lineColor: '#006DB7',
      ),
    );
  }

  Map<String, dynamic> getLinesMap(List<LatLng> latlngs) {
    return {
      "name": "FeatureCollection",
      "features": [
        {"type": "Feature"}
      ]
    };
  }

  Future<void> addImageFromAsset(String name, String assetName) async {
    final ByteData bytes = await rootBundle.load(assetName);
    final Uint8List list = bytes.buffer.asUint8List();
    return mapController!.addImage(name, list);
  }

  void _onStyleLoaded() async {
    await addImageFromAsset('poi', 'assets/images/poi.png');
    if (floorplans != null) {
      _changeFloorPlanImage(floorplans![currentFloor]);
    }
    mapController!.onSymbolTapped.add(_onSymbolTapped);
  }

  void _onSymbolTapped(Symbol symbol) {
    // final gj = symbol.toGeoJson();
    // log('$gj', name: 'GeoJSON');
    final lat = symbol.data!['geometry']['coordinates'][0] as double,
        lng = symbol.data!['geometry']['coordinates'][1] as double;
    log("STARTWAYFINDING: $symbol");
    IndoorAtlas.startWayfindingTo(
      IAWayfindingRequest(
        destination: IACoordinate(lat, lng),
        floor: currentFloor,
      ),
    );
    log('${symbol.toGeoJson()}');
  }

  List<LatLng> _getRouteLatLng(IARoute route) {
    List<LatLng> geometry = [];
    for (var leg in route.legs) {
      geometry
        ..add(
          LatLng(leg.begin.coordinate.latitude, leg.begin.coordinate.longitude),
        )
        ..add(
          LatLng(leg.end.coordinate.latitude, leg.end.coordinate.longitude),
        );
    }
    return geometry;
  }
}
