import 'package:flutter/material.dart';

import '../../data/ia/indooratlas.dart';
import '../../data/models/models.dart';
import '../../utils/ia_callbacklistner.dart';

class IndoorAtlasListener extends StatefulWidget {
  final Widget child;
  final IACallbackListener listener;
  final bool enabled;
  IndoorAtlasListener({
    Key? key,
    required String name,
    this.enabled = true,
    this.child = const SizedBox.shrink(),
    IAOnStatusCb? onStatus,
    ValueSetter<IALocation>? onLocation,
    IAOnVenueCb? onVenue,
    IAOnFloorplanCb? onFloorplan,
    IAOnWayfindingRouteCb? onWayfindingRoute,
    IAOnOrientationCb? onOrientation,
    ValueSetter<double>? onHeading,
  })  : listener = IACallbackListener(
          name: name,
          onStatusCb: onStatus,
          onLocationCb: onLocation,
          onVenueCb: onVenue,
          onFloorplanCb: onFloorplan,
          onWayfindingRouteCb: onWayfindingRoute,
          onOrientationCb: onOrientation,
          onHeadingCb: onHeading,
        ),
        super(key: key);
  @override
  State<StatefulWidget> createState() => _IndoorAtlasListenerState();
}

class _IndoorAtlasListenerState extends State<IndoorAtlasListener> {
  void _enable(IAListener? old) {
    if (widget.enabled) {
      if (old != null) {
        IndoorAtlas.resubscribe(old, widget.listener);
      } else {
        IndoorAtlas.subscribe(widget.listener);
      }
    } else if (old != null) {
      IndoorAtlas.unsubscribe(old);
    }
  }

  @override
  void initState() {
    super.initState();
    _enable(null);
  }

  @override
  void didUpdateWidget(IndoorAtlasListener old) {
    super.didUpdateWidget(old);
    _enable(old.listener);
  }

  @override
  void dispose() {
    IndoorAtlas.unsubscribe(widget.listener);
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return widget.child;
  }
}
