import 'package:flutter/foundation.dart';

import '../data/models/models.dart';

typedef IAOnStatusCb = void Function(IAStatus status, String message);
typedef IAOnVenueCb = void Function(bool enter, IAVenue venue);
typedef IAOnFloorplanCb = void Function(bool enter, IAFloorplan floorplan);
typedef IAOnWayfindingRouteCb = void Function(
    IARoute route, IAWayfindingRequest? request);
typedef IAOnOrientationCb = void Function(
    double x, double y, double z, double w);

class IACallbackListener extends IAListener {
  final IAOnStatusCb? onStatusCb;
  final ValueSetter<IALocation>? onLocationCb;
  final IAOnVenueCb? onVenueCb;
  final IAOnFloorplanCb? onFloorplanCb;
  final IAOnWayfindingRouteCb? onWayfindingRouteCb;
  final IAOnOrientationCb? onOrientationCb;
  final ValueSetter<double>? onHeadingCb;
  IACallbackListener({
    required String name,
    required this.onStatusCb,
    required this.onLocationCb,
    required this.onVenueCb,
    required this.onFloorplanCb,
    required this.onWayfindingRouteCb,
    required this.onOrientationCb,
    required this.onHeadingCb,
  }) : super(name);

  @override
  void onStatus(IAStatus status, String message) {
    onStatusCb?.call(status, message);
  }

  @override
  void onLocation(IALocation position) {
    onLocationCb?.call(position);
  }

  @override
  void onVenue(bool enter, IAVenue venue) {
    onVenueCb?.call(enter, venue);
  }

  @override
  void onFloorplan(bool enter, IAFloorplan floorplan) {
    onFloorplanCb?.call(enter, floorplan);
  }

  @override
  void onWayfindingRoute(IARoute route, IAWayfindingRequest? request) {
    onWayfindingRouteCb?.call(route, request);
  }

  @override
  void onOrientation(double x, double y, double z, double w) {
    onOrientationCb?.call(x, y, z, w);
  }

  @override
  void onHeading(double heading) {
    onHeadingCb?.call(heading);
  }
}